'''module for calculating accommodation coefficients, automatically generated by kimt_prep'''

# code that expresses and performs the functions for accommodation 
# coefficients that are given by the user in the model variables file 
# and that are intended to be calculated real-time via the kimt_calc 
# module 

def accom_coeff_func(accom_coeff, radius):

	# ------------------------------------------------------------------ 
	# inputs:
	# accom_coeff - array containing accommdotion coefficients for all 
	# components
	# radius - radii of size bins (m)
	# ------------------------------------------------------------------ 

	# calculate accommodation coefficients 

	return(accom_coeff)
